#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
###############################################################################
#
# Copyright (C) 2015, 2016, 2017 Daniel Rodriguez
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import collections
import json
import threading
import uuid
from datetime import date, datetime, timedelta

from aq_lib.model.exec_type import ExecType
from aq_lib.model.order_status import OrderStatus
from aq_lib.utils.logger import logger
from backtrader import (num2date, BrokerBase, Order, OrderBase)
from backtrader.comminfo import CommInfoBase
from backtrader.utils.py3 import bytes, bstr, with_metaclass, queue

from arquants.definitions import aqhandler
from arquants import eventFactory
from arquants.stores import aq_store

bytes = bstr  # py2/3 need for ibpy


class Order(OrderBase):
    """Subclasses the IBPy order to provide the minimum extra functionality
    needed to be compatible with the internally defined orders

    Once ``OrderBase`` has processed the parameters, the __init__ method takes
    over to use the parameter values and set the appropriate values in the
    ib.ext.Order.Order object

    Any extra parameters supplied with kwargs are applied directly to the
    ib.ext.Order.Order object, which could be used as follows::

      Example: if the 4 order execution types directly supported by
      ``backtrader`` are not enough, in the case of for example
      *Interactive Brokers* the following could be passed as *kwargs*::

        orderType='LIT', lmtPrice=10.0, auxPrice=9.8

      This would override the settings created by ``backtrader`` and
      generate a ``LIMIT IF TOUCHED`` order with a *touched* price of 9.8
      and a *limit* price of 10.0.

    This would be done almost always from the ``Buy`` and ``Sell`` methods of
    the ``Strategy`` subclass being used in ``Cerebro``
    """

    def __str__(self):
        """Get the printout from the base class and add some ib.Order specific
        fields"""
        basetxt = super(Order, self).__str__()
        tojoin = [basetxt]
        tojoin.append('Ref: {}'.format(self.ref))
        tojoin.append('orderId: {}'.format(self.m_orderId))
        tojoin.append('Action: {}'.format(self.m_action))
        tojoin.append('Size (ib): {}'.format(self.m_totalQuantity))
        tojoin.append('Lmt Price: {}'.format(self.m_lmtPrice))
        tojoin.append('Aux Price: {}'.format(self.m_auxPrice))
        tojoin.append('OrderType: {}'.format(self.m_orderType))
        tojoin.append('Tif (Time in Force): {}'.format(self.m_tif))
        tojoin.append('GoodTillDate: {}'.format(self.m_goodTillDate))
        return '\n'.join(tojoin)

    # Map backtrader order types to the ib specifics
    _IBOrdTypes = {
        None: bytes('MKT'),  # default
        Order.Market: bytes('MKT'),
        Order.Limit: bytes('LMT'),
        Order.Close: bytes('MOC'),
        Order.Stop: bytes('STP'),
        Order.StopLimit: bytes('STPLMT'),
        Order.StopTrail: bytes('TRAIL'),
        Order.StopTrailLimit: bytes('TRAIL LIMIT'),
    }

    (MarketToLimit) = range(8, 9)

    def __init__(self, action, **kwargs):

        # Marker to indicate an openOrder has been seen with
        # PendinCancel/Cancelled which is indication of an upcoming
        # cancellation

        self._willexpire = False

        self.ordtype = self.Buy if action == 'BUY' else self.Sell

        super(Order, self).__init__()

        # Now fill in the specific IB parameters
        self.m_orderType = self._IBOrdTypes[self.exectype]
        self.m_permid = 0

        # 'B' or 'S' should be enough
        self.m_action = bytes(action)

        # Set the prices
        self.m_lmtPrice = 0.0
        self.m_auxPrice = 0.0

        if self.exectype == self.Market:  # is it really needed for Market?
            pass
        elif self.exectype == self.Close:  # is it ireally needed for Close?
            pass
        elif self.exectype == self.Limit:
            self.m_lmtPrice = self.price
        elif self.exectype == self.Stop:
            self.m_auxPrice = self.price  # stop price / exec is market
        elif self.exectype == self.StopLimit:
            self.m_lmtPrice = self.pricelimit  # req limit execution
            self.m_auxPrice = self.price  # trigger price
        elif self.exectype == self.StopTrail:
            if self.trailamount is not None:
                self.m_auxPrice = self.trailamount
            elif self.trailpercent is not None:
                # value expected in % format ... multiply 100.0
                self.m_trailingPercent = self.trailpercent * 100.0
        elif self.exectype == self.StopTrailLimit:
            self.m_trailStopPrice = self.m_lmtPrice = self.price
            # The limit offset is set relative to the price difference in TWS
            self.m_lmtPrice = self.pricelimit
            if self.trailamount is not None:
                self.m_auxPrice = self.trailamount
            elif self.trailpercent is not None:
                # value expected in % format ... multiply 100.0
                self.m_trailingPercent = self.trailpercent * 100.0

        self.m_totalQuantity = self.abs_size = abs(self.size)  # ib takes only positives

        self.m_transmit = self.transmit
        if self.parent is not None:
            self.m_parentId = self.parent.m_orderId

        # Time In Force: DAY, GTC, IOC, GTD
        if self.valid is None:
            tif = 'GTC'  # Good til cancelled
        elif isinstance(self.valid, (datetime, date)):
            tif = 'GTD'  # Good til date
            self.m_goodTillDate = bytes(self.valid.strftime('%Y%m%d %H:%M:%S'))
        elif isinstance(self.valid, (timedelta,)):
            if self.valid == self.DAY:
                tif = 'DAY'
            else:
                tif = 'GTD'  # Good til date
                valid = datetime.now() + self.valid  # .now, using localtime
                self.m_goodTillDate = bytes(valid.strftime('%Y%m%d %H:%M:%S'))

        elif self.valid == 0:
            tif = 'DAY'
        else:
            tif = 'GTD'  # Good til date
            valid = num2date(self.valid)
            self.m_goodTillDate = bytes(valid.strftime('%Y%m%d %H:%M:%S'))

        self.m_tif = bytes(tif)

        # OCA
        self.m_ocaType = 1  # Cancel all remaining orders with block

        # pass any custom arguments to the order
        for k in kwargs:
            setattr(self, (not hasattr(self, k)) * 'm_' + k, kwargs[k])


class AQCommInfo(CommInfoBase):
    """
    Commissions are calculated by ib, but the trades calculations in the
    ```Strategy`` rely on the order carrying a CommInfo object attached for the
    calculation of the operation cost and value.

    These are non-critical informations, but removing them from the trade could
    break existing usage and it is better to provide a CommInfo objet which
    enables those calculations even if with approvimate values.

    The margin calculation is not a known in advance information with IB
    (margin impact can be gotten from OrderState objects) and therefore it is
    left as future exercise to get it"""

    def getvaluesize(self, size, price):
        # In real life the margin approaches the price
        return abs(size) * price

    def getoperationcost(self, size, price):
        """Returns the needed amount of cash an operation would cost"""
        # Same reasoning as above
        return abs(size) * price


class MetaAQBroker(BrokerBase.__class__):
    def __init__(cls, name, bases, dct):
        """Class has already been created ... register"""
        # Initialize the class
        super(MetaAQBroker, cls).__init__(name, bases, dct)
        aq_store.AQStore.BrokerCls = cls


class AQBroker(with_metaclass(MetaAQBroker, BrokerBase)):
    """Broker implementation for Interactive Brokers.

    This class maps the orders/positions from Interactive Brokers to the
    internal API of ``backtrader``.

    Notes:

      - ``tradeid`` is not really supported, because the profit and loss are
        taken directly from IB. Because (as expected) calculates it in FIFO
        manner, the pnl is not accurate for the tradeid.

      - Position

        If there is an open position for an asset at the beginning of
        operaitons or orders given by other means change a position, the trades
        calculated in the ``Strategy`` in cerebro will not reflect the reality.

        To avoid this, this broker would have to do its own position
        management which would also allow tradeid with multiple ids (profit and
        loss would also be calculated locally), but could be considered to be
        defeating the purpose of working with a live broker
    """
    params = ()

    def __init__(self, **kwargs):
        super(AQBroker, self).__init__()

        self.ib = aq_store.AQStore(**kwargs)

        self.startingcash = self.cash = 0.0
        self.startingvalue = self.value = 0.0

        self._lock_orders = threading.Lock()  # control access
        self._lock_ers = threading.Lock()  # control access
        self.orderbyid = dict()  # orders by order id
        self.executions = dict()  # notified executions
        self.ordstatus = collections.defaultdict(dict)
        self.notifs = queue.Queue()  # holds orders which are notified
        self.tonotify = collections.deque()  # hold oids to be notified
        self.ers = queue.Queue()

    def start(self):
        super(AQBroker, self).start()
        self.ib.start(broker=self)

        # if self.ib.connected():
        #   self.ib.reqAccountUpdates()
        #  self.startingcash = self.cash = self.ib.get_acc_cash()
        # self.startingvalue = self.value = self.ib.get_acc_value()
        # else:
        self.startingcash = self.cash = 0.0
        self.startingvalue = self.value = 0.0

    def stop(self):
        super(AQBroker, self).stop()
        self.ib.stop()

    def getcash(self):
        # This call cannot block if no answer is available from ib
        # self.cash = self.ib.get_acc_cash()
        return self.cash

    def getvalue(self, datas=None):
        # self.value = self.ib.get_acc_value()
        return self.value

    def getposition(self, data, clone=True):
        return self.ib.getposition(data.tradecontract, clone=clone)

    def manual_cancel(self, order_id):
        try:
            o = self.orderbyid[order_id]
            self.cancel(o)
        except (ValueError, KeyError):
            logger.debug("manual_cancel: unknown order {}".format(order_id))
            return  # not found ... not cancellable

    def cancel(self, order):
        try:
            self.orderbyid[order.m_orderId]
        except (ValueError, KeyError):
            logger.debug("cancel: unknown order {}".format(order.m_orderId))
            return  # not found ... not cancellable
        if order.status == Order.Cancelled:  # already cancelled
            return
        self.ib.cancel_order(order)

    def mass_cancel(self, routing_key=None, segment=None, account=None):
        self.ib.mass_cancel(routing_key=routing_key, segment=segment, account=account)

    def orderstatus(self, order):
        try:
            o = self.orderbyid[order.m_orderId]
        except (ValueError, KeyError):
            o = order

        return o.status

    def send_bulk_new_orders(self, orders):
        self.ib.send_bulk_new_orders(orders)

    def submit(self, order):
        order.submit(self)

        # ocoize if needed. One cancel the other order
        if order.oco is None:  # Generate a UniqueId
            order.m_ocaGroup = bytes(uuid.uuid4())
        else:
            order.m_ocaGroup = self.orderbyid[order.oco.m_orderId].m_ocaGroup

        self.orderbyid[order.m_orderId] = order
        self.ib.place_order(order)
        self.notify(order)

        return order

    @staticmethod
    def getcommissioninfo(data):
        # contract = data.tradecontract
        # try:
        #   mult = float(contract.m_multiplier)
        # except (ValueError, TypeError):
        mult = 1.0
        # stocklike = contract.m_secType not in ('FUT', 'OPT', 'FOP',)

        return AQCommInfo(mult=mult, stocklike=True)

    def _makeorder(self, action, owner, data,
                   size, price=None, plimit=None,
                   exectype=None, valid=None,
                   tradeid=0, **kwargs):

        order = Order(action, owner=owner, data=data,
                        size=size, price=price, pricelimit=plimit,
                        exectype=exectype, valid=valid,
                        tradeid=tradeid,
                        clientId=self.ib.clientId,
                        orderId=self.ib.generate_id(),
                        **kwargs)

        order.addcomminfo(self.getcommissioninfo(data))
        return order

    def buy(self, owner, data,
            size, price=None, plimit=None,
            exectype=None, valid=None, tradeid=0,
            **kwargs):

        order = self._makeorder(
            'BUY',
            owner, data, size, price, plimit, exectype, valid, tradeid,
            **kwargs)

        return self.submit(order)

    def sell(self, owner, data,
             size, price=None, plimit=None,
             exectype=None, valid=None, tradeid=0,
             **kwargs):

        order = self._makeorder(
            'SELL',
            owner, data, size, price, plimit, exectype, valid, tradeid,
            **kwargs)

        return self.submit(order)

    def replace(self, owner, size, price=None, order=None, **kwargs):
        neworder = self._makeorder('BUY' if order.ordtype == 0 else 'SELL', owner, order.data, size, price,
                                   exectype=order.exectype, originalOrderId=order.m_serverOrderId,
                                   originalClientOrderId=order.m_orderId, **kwargs)
        return self.submit(neworder)

    def notify(self, order):
        self.notifs.put(order.clone())

    def get_notification(self):
        try:
            return self.notifs.get(False)
        except queue.Empty:
            pass

        return None

    def next(self):
        self.notifs.put(None)  # mark notificatino boundary

    def update_order(self, order, ex):
        notify = False
        if ex.orderStatus == OrderStatus.New.value:
            if order.status != Order.Accepted:
                order.m_serverOrderId = ex.orderID
                order.accept(self)
                notify = True
        elif ex.orderStatus == OrderStatus.Canceled.value:
            if order.status != Order.Canceled:
                order.cancel()
                order.m_serverOrderId = ex.orderID
                notify = True
        elif ex.orderStatus == OrderStatus.Rejected.value:
            if order.status != Order.Rejected:
                order.reject()
                notify = True
        elif ex.orderStatus == OrderStatus.Replaced.value:
            order.m_serverOrderId = ex.orderID
            order.cancel()
            notify = True
        elif ex.orderStatus == OrderStatus.Expired.value:
            if order.status != Order.Expired:
                order.m_serverOrderId = ex.orderID
                order.expire()
                notify = True
        elif ex.orderStatus == OrderStatus.PartiallyFilled.value:
            if order.status != Order.Partial:
                order.m_serverOrderId = ex.orderID
                order.partial()
                notify = True
        elif ex.orderStatus == OrderStatus.Filled.value:
            if order.status != Order.Completed:
                order.m_serverOrderId = ex.orderID
                order.completed()
                notify = True
        if notify:
            self.notify(order)
        # En varios casos queda pendiente actualizar la posicion, pero eso va a ser algo que se va a realizar
        # en el modulo de riesgo

    def push_execution(self, ex):
        with self._lock_orders:
            order = None
            original_order = None
            try:
                order = self.orderbyid[ex.clientOrderID]
            except KeyError:
                logger.debug("Debe ser un cancel")

            if ex.execType in [ExecType.Canceled.value]:
                if ex.originalClientOrderId is not None:
                    original_order = self.orderbyid[ex.originalClientOrderId]
                    original_order.cancel()
                    self.notify(original_order)
                elif order is not None:
                    order.cancel()
                    self.notify(order)
            elif ex.execType in [ExecType.Replaced.value]:
                original_order = self.orderbyid[ex.originalClientOrderId]
                original_order.cancel()
                if ex.orderStatus in (OrderStatus.New.value, OrderStatus.PartiallyFilled.value):
                    order.m_serverOrderId = ex.orderID
                    order.accept(self)
                    self.notify(order)
                self.notify(original_order)

            elif ex.execType == ExecType.New.value:
                if not order.status == Order.Accepted:  # Duplicate detection
                    order.m_serverOrderId = ex.orderID
                    order.accept(self)
                    self.notify(order)

            elif ex.execType == ExecType.Rejected.value:
                if order is not None:
                    order.reject()
                    self.notify(order)
                else:
                    logger.warning("Received an OrderCancelReject - clientOrderId %s" % ex.clientOrderID)
            elif ex.execType == ExecType.Trade.value:
                order.m_serverOrderId = ex.orderID
                position = self.getposition(order.data, clone=False)
                # pprice_orig = position.price
                size = ex.lastQty if int(ex.side) == 1 else -ex.lastQty
                price = ex.lastPrice
                # use pseudoupdate and let the updateportfolio do the real update?
                psize, pprice, opened, closed = position.update(size, price)
                # split commission between closed and opened
                comm = 0
                closedcomm = comm * closed / size
                openedcomm = comm - closedcomm

                # comminfo = order.comminfo
                # closedvalue = comminfo.getoperationcost(closed, pprice_orig)
                # openedvalue = comminfo.getoperationcost(opened, price)

                # default in m_pnl is MAXFLOAT
                # pnl = ex.m_realizedPNL if closed else 0.0
                pnl = 0
                # The internal br
                # Use the actual time provided by the execution object
                # The report from TWS is in actual local time, not the data's tz
                dt = ex.transactTime

                # Need to simulate a margin, but it plays no role, because it is
                # controlled by a real broker. Let's set the price of the item
                margin = order.data.close[0]
                order.tradeid = ex.execID
                order.execute(dt, size, price,
                              closed, 0, closedcomm,
                              opened, 0, openedcomm,
                              margin, pnl,
                              psize, pprice)
                if ex.orderStatus == OrderStatus.Filled.value:
                    order.completed()
                elif ex.orderStatus == OrderStatus.PartiallyFilled.value:
                    if order.status != Order.Completed or order.staus not in (Order.Canceled, Order.Cancelled):
                        order.partial()
                self.notify(order)
                self.trade_executed(ex)
            elif ex.execType == ExecType.OrderStatus.value:
                self.update_order(order, ex)
            if order is None or not (order.status == Order.Completed and
                                     ex.orderStatus == OrderStatus.PartiallyFilled.value):
                self.publish_order_updated_event(ex, order)
            if original_order is not None:
                self.publish_order_updated_event_from_order(original_order)

    def publish_order_updated_event(self, ex, order):
        aqhandler.send(json.dumps(eventFactory.createOrderUpdatedEventFromExecReport(ex, order).to_dict()),
                       self.ib.get_delivery_rk())

    def publish_order_updated_event_from_order(self, original_order):
        aqhandler.send(json.dumps(eventFactory.createOrderUpdatedEventFromOrder(original_order).to_dict()),
                       self.ib.get_delivery_rk())

    def trade_executed(self, ex):
        aqhandler.send(json.dumps(eventFactory.createTradeExecutedEvent(ex).to_dict()), self.ib.get_delivery_rk())

    def cancel_all(self):
        for key in self.orderbyid:
            order = self.orderbyid[key]
            if order.status == Order.Accepted or order.status == Order.Partial:
                order.cancel()
                self.ib.cancel_order(order)



import traceback
from multiprocessing import Process
import gspread
import gspread.utils
import gspread.utils
from aq_lib.utils.logger import logger
from gspread import WorksheetNotFound


class SheetLogger(Process):
    # scope = ["https://spreadsheets.google.com/feeds",'https://www.googleapis.com/auth/spreadsheets',"https://www.googleapis.com/auth/drive.file","https://www.googleapis.com/auth/drive"]
    #
    # def __init__(self,google_json=None, sheet_id=None,queue=None):
    #     self.logger_queue = queue
    #     google_json = google_json.replace("\n","\\n")
    #     google_dict = json.loads(google_json)
    #     self.credentials = ServiceAccountCredentials.from_json_keyfile_dict(google_dict, self.scope)
    #     self.client = gspread.authorize(self.credentials)
    #     if sheet_id is not None:
    #         self.spreadsheet = self.client.open(sheet_id)

    def __init__(self, client=None, spreadsheet=None, queue=None):
        super(SheetLogger, self).__init__()
        self.client = client
        self.spreadsheet = spreadsheet
        self.logger_queue = queue

    #def open_spreadsheet(self,sheet_id):
     #   self.spreadsheet = self.client.open(sheet_id)

    def run(self):
        while True:
            # item =  self.logger_queue.get()
            # self.process_item(item)
            pass

    def process_item(self, item):
        try:
            self.client.login()  # TODO revisar si es neceario
            try:
                sheet = self.spreadsheet.worksheet(item.worksheet_name)
            except WorksheetNotFound:
                sheet = self.spreadsheet.add_worksheet(title=item.worksheet_name, rows=1, cols=100)

            key,values = self.recursive_items(item.values)

            if item.cell is not None:
                cells = self.get_cell(sheet, item.cell, len(item.values))
                for i, cell in enumerate(cells):
                    cell.value = values[i]
                sheet.update_cells(cells)
            else:
                row_count = sheet.row_count
                sheet.insert_row(values, row_count)
        except Exception:
            logger.warning("Exception loggin to sheets")
            logger.warning(traceback.format_exc())

    def get_cell(self, sheet, cell, qty):
        coords = gspread.utils.a1_to_rowcol(cell)
        range = cell + ":" + gspread.utils.rowcol_to_a1(coords[0], coords[1] + qty-1)
        return sheet.range(range)

    @staticmethod
    def recursive_items(dictionary):
        keys = []
        values = []
        for key, value in dictionary.items():
            if type(value) is dict:
                for k1, v1 in value.items():
                    keys.append(k1)
                    values.append(v1)
            else:
                keys.append(key)
                values.append(value)
        return keys, values






import json

from backtrader import Position

from arquants.definitions import aqhandler
from arquants import eventFactory
from arquants.definitions import force_async


class PositionCalculator(Position):

    def __init__(self):
        super(PositionCalculator, self).__init__()
        self.last_px = 0
        self.close_position = 0
        self.symbol = None
        self.changed = False

    def update(self, size, price, dt=None):
        self.changed = True
        new_size, new_avg_price, _open, close = super(PositionCalculator, self).update(size, price)
        if close != 0:
            if size < 0:
                close = -close
            else:
                close = abs(close)
            self.close_position += (price - self.price_orig) * close
        return new_size, new_avg_price, _open, close

    def get_position(self):
        return {"symbol": self.symbol, "size": self.size, "price": self.price,
                "open_position": self.get_open_position(), "close_position": self.close_position}

    def get_open_position(self):
        if self.last_px != 0:
            return (self.last_px - self.price) * self.size
        else:
            return 0

    def update_on_md(self, symbol, last_px):
        if self.last_px != last_px:
            self.last_px = last_px
            self.changed = True
        if self.symbol is None:
            self.symbol = symbol
        if self.changed:
            self.publish_position()
            self.changed = False

    @force_async
    def publish_position(self):
        aqhandler.send(json.dumps(eventFactory.createPositionUpdatedEvent(self.get_position()).to_dict()), "worker")



import json

from backtrader import Observer
from arquants.definitions import aqhandler
from arquants.event.event_factory import eventFactory


class BacktestingObserver(Observer):

    def __init__(self, *args, **kwargs):
        self.strategy_id = kwargs.get('strategy_id', None)
        self.strategy_version = kwargs.get('strategy_version', None)
        self.execution_id = kwargs.get('execution_id', None)
        self.arquants_user = kwargs.get('arquants_user', None)
        self.delivery_routing_key = kwargs.get('delivery_routing_key', None)
        self.execution_type = kwargs.get('execution_type', None)
        super(BacktestingObserver, self).__init__(self, *args, **kwargs)

    # No storeo nada en la estrategia
    plotinfo = dict(plot=False, subplot=False, plotlinelabels=False)
    _stclock = False

    # requerido por cerebrodockerfi
    lines = ('',)

    def next(self):
        positions = dict()
        for data in self._owner.datas:
            positions[data.tradecontract] = self._owner.broker.getposition(data, True).get_position()
        aqhandler.send(json.dumps(eventFactory.createPositionUpdatedEvent(positions).to_dict()),
                       self.delivery_routing_key)



btstatustranslator = {
    'Accepted': '0',
    'Partial': '1',
    'Completed': '2',
    'DoneForDay': '3',
    'Canceled': '4',
    'Replaced': '5',
    'PendingCancel': '6',
    'Stopped': '7',
    'Rejected': '8',
    'Suspended': '9',
    'Submitted': 'A',
    'Calculated': 'B',
    'Expired': 'C',
    'AcceptedForBidding': 'D',
    'PendingReplace': 'E',
}



from aq_lib.utils.logger import logger
from gspread import WorksheetNotFound
from gspread.exceptions import APIError
import gspread


class SheetReader:

    def __init__(self, credentials, spreadsheet_id):
        client = gspread.authorize(credentials)
        try:
            self.spreadsheet = client.open_by_key(spreadsheet_id)
        except Exception as e:
            logger.warning("Spreadsheet not found")

    def row_values(self, sheet_name=None, row=None):
        if sheet_name is not None and row is not None:
            try:
                sheet = self.spreadsheet.worksheet(sheet_name)
                return sheet.row_values(row)
            except WorksheetNotFound as e:
                logger.warning("Intentando abrir una hoja que no existe")
            except APIError as e:
                logger.warning("Error intentando leer un spreadsheet")
        else:
            logger.warning("Sheet name or row not specified")

    def col_values(self, sheet_name = None, col=None):
        if sheet_name is not None and col is not None:
            try:
                sheet = self.spreadsheet.worksheet(sheet_name)
                return sheet.col_values(col)
            except WorksheetNotFound as e:
                logger.warning("Intentando abrir una hoja que no existe")
            except APIError as e:
                logger.warning("Error intentando leer un spreadsheet")
            except ValueError as e:
                logger.warning("Parámetro col debe ser int")
        else:
            logger.warning("Sheet name or row not specified")

    def get_all_values(self, sheet_name):
        if sheet_name is not None:
            try:
                sheet = self.spreadsheet.worksheet(sheet_name)
                return sheet.get_all_values()
            except WorksheetNotFound:
                logger.warning("Intentando abrir una hoja que no existe")
        else:
            logger.warning("Sheet name not specified")

    def get_all_records(self, sheet_name):
        if sheet_name is not None:
            try:
                logger.debug("Querying all records for %s" %sheet_name)
                sheet = self.spreadsheet.worksheet(sheet_name)
                return sheet.get_all_records()
            except WorksheetNotFound:
                logger.warning("Intentando abrir una hoja que no existe")
        else:
            logger.warning("Sheet name not specified")




from setuptools import setup, find_packages

__version__ = '0.0.25'

setup(
    name='arquants',
    version=__version__,
    description='arquants strategy engine',
    url='https://gitlab.com/arquants/algorithm-engine.git',
    author='TradeSpark',
    author_email='info@arquants.trading',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        "aq-lib==0.0.9",
        "amqp==2.2.2",
        "APScheduler==3.6.0",
        "dateparser==0.6.0",
        "enum34==1.1.6",
        "jsonpickle==0.9.5",
        "kombu==4.2.1",
        "logger==1.4",
        "marshmallow==3.0.0b11",
        "python-dateutil==2.6.1",
        "pytz==2017.3",
        "redis==2.10.6",
        "regex==2018.2.3",
        "ruamel.yaml==0.15.35",
        "tzlocal==1.5.1",
        "vine==1.1.4",
        "certifi==2019.6.16",
        "chardet==3.0.4",
        "gspread==3.1.0",
        "httplib2==0.13.1",
        "idna==2.8",
        "oauth2client==4.1.3",
        "pyasn1==0.4.5",
        "pyasn1-modules==0.2.5",
        "requests==2.22.0",
        "rsa==4.0",
        "urllib3==1.25.3",
        "simplejson",
        "websocket",
        "websocket-client",
        "google-api-python-client==1.9.3",
        "wheel==0.35.1",
        "backtrader==1.9.74.123"
    ],
)



from backtrader import Observer


class LiveTradingObserver(Observer):

    def __init__(self, *args, **kwargs):
        self.delivery_routing_key = kwargs.get('delivery_routing_key', None)
        super(LiveTradingObserver, self).__init__(self, *args, **kwargs)

    # No storeo nada en la estrategia
    plotinfo = dict(plot=False, subplot=False, plotlinelabels=False)
    _stclock = False

    # requerido por cerebrodockerfi
    lines = ('',)

    def next(self):
        positions = dict()
        for data in self._owner.datas:
            position = self._owner.broker.getposition(data, True)
            if position.updated:
                positions[data.tradecontract] = position.get_position()
                position.updated = False
        #  aqhandler.send(json.dumps(eventFactory.createPositionUpdatedEvent(positions).to_dict()),
                       #  self.delivery_routing_key)



from enum import Enum
from time import sleep
from multiprocessing import Queue
from arquants.cerebro import AQCerebro
from arquants.definitions import aqhandler
from arquants.stores.aq_store import AQStore


class MDSubsType(Enum):
    AGGREGATED = 'aggregated'
    DISAGGREGATED = 'disaggregated'
    TOP = 'top'


class Symbol:

    def __init__(self, market=None, symbol=None, md_type: MDSubsType = None):
        self.market = market
        self.symbol = symbol
        self.md_type = md_type


class Initiator:

    def __init__(self, symbols=None, strat_cls=None):
        aqhandler.init_public_aq()
        if not aqhandler.get_account():
            raise Exception("No account")

        queue = Queue()
        store = AQStore(queue)
        broker = store.getbroker()

        self.cerebro = AQCerebro(queue, quicknotify=True, stdstats=False)
        self.cerebro.setbroker(broker)

        for item in symbols:
            market = item.market
            symbol = item.symbol
            md_type = item.md_type

            data_cls = store.getdataclass(md_type.value)
            data = data_cls(dataname=symbol, account=aqhandler.get_account(), routingkey='rk-rofx-income',
                             market=market, price_size=1)

            self.cerebro.adddata(data)

        self.cerebro.addstrategy(strat_cls)

        # Le doy unos segundos para que se conecte a la api y al websocket.
        sleep(2)

    def start(self):
        sleep(2)
        self.cerebro.run()
